@@ -600,13 +600,15 @@ static void nis_freeentry_cols(entry_col *ec, int len)
 {
     int i;
 
-    
+
     if (ec == NULL)
 	return;
     
     for (i = 0; i < len; i++)
 	if (ec[i].ec_value.ec_value_val)
+	{
 	    free(ec[i].ec_value.ec_value_val);
+        }
 
     free(ec);
 }
@@ -811,6 +813,7 @@ void nis_freeattrs(nis_attr *at, int len)
     
     for (i = 0; i < len; i++)
 	nis_cleanattr(&at[i]);
+
 }
 
 void nis_freeattr(nis_attr *at)
@@ -1216,15 +1219,12 @@ nis_result *nis_dupresult(nis_result *res)
 
 void nis_cleanresult(nis_result *nsres)
 {
-    int i;
-
-
     if (nsres == NULL)
 	return;
 
     if (nsres->objects.objects_val)
-	for (i = 0; i < nsres->objects.objects_len; i++)
-	    nis_cleanobject(&nsres->objects.objects_val[i]);
+	nis_freeobjects(nsres->objects.objects_val,
+			nsres->objects.objects_len);
 
     nis_cleannetobj(&nsres->cookie);
 }
