@@ -57,7 +57,7 @@ int nis_setup(void)
 
 nis_name nis_splitname(const nis_name name, int *len, nis_attr **val)
 {
-    static nis_name rname;
+    static nis_name rname = NULL;
     char *cp, *tp1, *tp2, *tp3;
     int size;
 
@@ -172,7 +172,7 @@ nis_result *
 
 nis_result *nis_lookup(nis_name name, u_long flags)
 {
-    nis_result *res;
+    nis_result *res, *nres;
     struct ns_request nsreq;
     nis_name *names;
     int i;
@@ -215,7 +215,10 @@ nis_result *nis_lookup(nis_name name, u_long flags)
 	res = _nis_call(&nis_lookup_3, &nsreq);
     }
 
-    return nis_dupresult(res);
+    nres = nis_dupresult(res);
+    xdr_free(xdr_nis_result, (char *) res);
+    
+    return nres;
 }
 
 
@@ -264,6 +267,37 @@ struct ib_request *_nis_build_ib_request(const nis_name name,
 }
 
 
+static void _nis_clean_ib_request(struct ib_request *ibreq)
+{
+    if (ibreq->ibr_srch.ibr_srch_len)
+    {
+	nis_freeattrs(ibreq->ibr_srch.ibr_srch_val,
+		      ibreq->ibr_srch.ibr_srch_len);
+	free(ibreq->ibr_srch.ibr_srch_val);
+
+	ibreq->ibr_srch.ibr_srch_val = NULL;
+	ibreq->ibr_srch.ibr_srch_len = 0;
+    }
+
+    if (ibreq->ibr_name)
+    {
+	free(ibreq->ibr_name);
+	ibreq->ibr_name = NULL;
+    }
+
+    if (ibreq->ibr_cookie.n_bytes)
+    {
+	free(ibreq->ibr_cookie.n_bytes);
+	ibreq->ibr_cookie.n_bytes = NULL;
+	ibreq->ibr_cookie.n_len = 0;
+    }
+}
+
+
+/* Common save area for nis_first_entry()/nis_next_entry() */
+static nis_result *fn_res = NULL;
+
+
 nis_result *nis_first_entry(const nis_name name)
 {
     struct ib_request ibreq;
@@ -278,14 +312,15 @@ nis_result *nis_first_entry(const nis_name name)
 
     res = _nis_call(&nis_ibfirst_3, &ibreq);
 
-    if (ibreq.ibr_srch.ibr_srch_val)
-	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
-		      ibreq.ibr_srch.ibr_srch_len);
-
-    if (ibreq.ibr_name)
-	free(ibreq.ibr_name);
+    _nis_clean_ib_request(&ibreq);
     
-    return nis_dupresult(res);
+    if (fn_res)
+	nis_freeresult(fn_res);
+
+    fn_res = nis_dupresult(res);
+    xdr_free(xdr_nis_result, (char *) res);
+    
+    return fn_res;
 }
 
 
@@ -304,18 +339,19 @@ nis_result *nis_next_entry(const nis_name name, netobj *cookie)
       return NULL;
 
     if (cookie)
-      ibreq.ibr_cookie = *cookie;
+      nis_copynetobj(&ibreq.ibr_cookie, cookie);
 
     res = _nis_call(&nis_ibnext_3, &ibreq);
 
-    if (ibreq.ibr_srch.ibr_srch_val)
-	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
-		      ibreq.ibr_srch.ibr_srch_len);
+    _nis_clean_ib_request(&ibreq);
 
-    if (ibreq.ibr_name)
-	free(ibreq.ibr_name);
+    if (fn_res)
+	nis_freeresult(fn_res);
+
+    fn_res = nis_dupresult(res);
+    xdr_free(xdr_nis_result, (char *) res);
     
-    return nis_dupresult(res);
+    return fn_res;
 }
 
 
@@ -338,27 +374,33 @@ nis_result *nis_list(const nis_name name,
 
     res = _nis_call(&nis_iblist_3, &ibreq);
 
-    if (ibreq.ibr_srch.ibr_srch_val)
-	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
-		      ibreq.ibr_srch.ibr_srch_len);
+    _nis_clean_ib_request(&ibreq);
+    
+    if (fn_res)
+	nis_freeresult(fn_res);
 
-    if (ibreq.ibr_name)
-	free(ibreq.ibr_name);
+    fn_res = nis_dupresult(res);
+    xdr_free(xdr_nis_result, (char *) res);
     
     if (callback != NULL)
     {
+
+	/* XXX Poor mans callback emulation. Should use a real callback SVC */
+    
 	int i;
 
 
-	if (res->status > 1)
-	    return res;
+	if (fn_res->status > 1)
+	    return fn_res;
 
-	for (i = 0; i < res->objects.objects_len; i++)
-	    if ((*callback)(name, &res->objects.objects_val[i], userdata) != 0)
-		return res;
+	for (i = 0; i < fn_res->objects.objects_len; i++)
+	    if ((*callback)(name,
+			    &fn_res->objects.objects_val[i],
+			    userdata) != 0)
+		return fn_res;
     }
 
-    return res;
+    return fn_res;
 }
 
 
