@@ -50,7 +50,7 @@ struct group *dns_getgrent(void)
 }
 
 
-struct group *dns_getgrgid(int gid)
+struct group *dns_getgrgid(gid_t gid)
 {
     errno = ECONNREFUSED;
     return NULL;
