@@ -50,7 +50,7 @@ struct group *nis_getgrent(void)
 }
 
 
-struct group *nis_getgrgid(int gid)
+struct group *nis_getgrgid(gid_t gid)
 {
     errno = ECONNREFUSED;
     return NULL;
