@@ -27,7 +27,7 @@
 
 #include <stdio.h>
 #include <errno.h>
-#include "nis_hosts.h"
+#include <netdb/hosts.h>
 
 
 
