@@ -27,7 +27,7 @@
 
 #include <stdio.h>
 #include <errno.h>
-#include "nis_services.h"
+#include <netdb/services.h>
 
 
 void nis_setservent(int stayopen)
