@@ -30,8 +30,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <netdb/services.h>
 #include "yp_misc.h"
-#include "yp_services.h"
 #include "rpcsvc/ypclnt.h"
 
 #include "xalloc.h"
