@@ -31,10 +31,16 @@ extern void print_passwd(struct passwd *);
 int main(int argc, char *argv[])
 {
     struct passwd *pw;
+    int i, times;
 
+    if (argc > 2)
+	times = atoi(argv[2]);
+    else
+	times = 1;
 
-    pw = nis_getpwnam(argv[1]);
-
+    for (i = 0; i < times; i++)
+	pw = nis_getpwnam(argv[1]);
+	
     if (pw)
 	print_passwd(pw);
     else
